@@ -417,7 +417,6 @@ tp_button_handle_event(struct tp_dispatch *tp,
 		       enum button_event event,
 		       uint64_t time)
 {
-	struct libinput *libinput = tp_libinput_context(tp);
 	enum button_state current = t->button.state;
 
 	switch(t->button.state) {
@@ -445,11 +444,11 @@ tp_button_handle_event(struct tp_dispatch *tp,
 	}
 
 	if (current != t->button.state)
-		log_debug(libinput,
-			  "button state: from %s, event %s to %s\n",
-			  button_state_to_str(current),
-			  button_event_to_str(event),
-			  button_state_to_str(t->button.state));
+		evdev_log_debug(tp->device,
+				"button state: from %s, event %s to %s\n",
+				button_state_to_str(current),
+				button_event_to_str(event),
+				button_state_to_str(t->button.state));
 }
 
 void
@@ -503,14 +502,13 @@ tp_process_button(struct tp_dispatch *tp,
 		  const struct input_event *e,
 		  uint64_t time)
 {
-	struct libinput *libinput = tp_libinput_context(tp);
 	uint32_t mask = 1 << (e->code - BTN_LEFT);
 
 	/* Ignore other buttons on clickpads */
 	if (tp->buttons.is_clickpad && e->code != BTN_LEFT) {
-		log_bug_kernel(libinput,
-			       "received %s button event on a clickpad\n",
-			       libevdev_event_code_get_name(EV_KEY, e->code));
+		evdev_log_bug_kernel(tp->device,
+				     "received %s button event on a clickpad\n",
+				     libevdev_event_code_get_name(EV_KEY, e->code));
 		return;
 	}
 
@@ -627,7 +625,7 @@ tp_init_top_softbuttons(struct tp_dispatch *tp,
 static inline uint32_t
 tp_button_config_click_get_methods(struct libinput_device *device)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct tp_dispatch *tp = (struct tp_dispatch*)evdev->dispatch;
 	uint32_t methods = LIBINPUT_CONFIG_CLICK_METHOD_NONE;
 
@@ -672,7 +670,7 @@ static enum libinput_config_status
 tp_button_config_click_set_method(struct libinput_device *device,
 				  enum libinput_config_click_method method)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct tp_dispatch *tp = (struct tp_dispatch*)evdev->dispatch;
 
 	tp->buttons.click_method = method;
@@ -684,7 +682,7 @@ tp_button_config_click_set_method(struct libinput_device *device,
 static enum libinput_config_click_method
 tp_button_config_click_get_method(struct libinput_device *device)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct tp_dispatch *tp = (struct tp_dispatch*)evdev->dispatch;
 
 	return tp->buttons.click_method;
@@ -715,7 +713,7 @@ tp_click_get_default_method(struct tp_dispatch *tp)
 static enum libinput_config_click_method
 tp_button_config_click_get_default_method(struct libinput_device *device)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct tp_dispatch *tp = (struct tp_dispatch*)evdev->dispatch;
 
 	return tp_click_get_default_method(tp);
@@ -750,7 +748,7 @@ static enum libinput_config_status
 tp_clickpad_middlebutton_set(struct libinput_device *device,
 		     enum libinput_config_middle_emulation_state enable)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 
 	switch (enable) {
 	case LIBINPUT_CONFIG_MIDDLE_EMULATION_ENABLED:
@@ -832,7 +830,6 @@ void
 tp_init_buttons(struct tp_dispatch *tp,
 		struct evdev_device *device)
 {
-	struct libinput *libinput = tp_libinput_context(tp);
 	struct tp_touch *t;
 	const struct input_absinfo *absinfo_x, *absinfo_y;
 
@@ -844,15 +841,13 @@ tp_init_buttons(struct tp_dispatch *tp,
 	if (libevdev_has_event_code(device->evdev, EV_KEY, BTN_MIDDLE) ||
 	    libevdev_has_event_code(device->evdev, EV_KEY, BTN_RIGHT)) {
 		if (tp->buttons.is_clickpad)
-			log_bug_kernel(libinput,
-				       "%s: clickpad advertising right button\n",
-				       device->devname);
+			evdev_log_bug_kernel(device,
+					     "clickpad advertising right button\n");
 	} else if (libevdev_has_event_code(device->evdev, EV_KEY, BTN_LEFT) &&
 		   !tp->buttons.is_clickpad &&
 		   libevdev_get_id_vendor(device->evdev) != VENDOR_ID_APPLE) {
-			log_bug_kernel(libinput,
-				       "%s: non clickpad without right button?\n",
-				       device->devname);
+			evdev_log_bug_kernel(device,
+					     "non clickpad without right button?\n");
 	}
 
 	absinfo_x = device->abs.absinfo_x;
