@@ -143,9 +143,9 @@ tp_fake_finger_count(struct tp_dispatch *tp)
 	 * time */
 	if (__builtin_popcount(
 		       tp->fake_touches & ~(FAKE_FINGER_OVERFLOW|0x1)) > 1)
-	    log_bug_kernel(tp_libinput_context(tp),
-			   "Invalid fake finger state %#x\n",
-			   tp->fake_touches);
+		evdev_log_bug_kernel(tp->device,
+				     "Invalid fake finger state %#x\n",
+				     tp->fake_touches);
 
 	if (tp->fake_touches & FAKE_FINGER_OVERFLOW)
 		return FAKE_FINGER_OVERFLOW;
@@ -214,6 +214,7 @@ tp_new_touch(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 	tp_motion_history_reset(t);
 	t->dirty = true;
 	t->has_ended = false;
+	t->was_down = false;
 	t->state = TOUCH_HOVERING;
 	t->pinned.is_pinned = false;
 	t->millis = time;
@@ -226,6 +227,7 @@ tp_begin_touch(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 	t->dirty = true;
 	t->state = TOUCH_BEGIN;
 	t->millis = time;
+	t->was_down = true;
 	tp->nfingers_down++;
 	t->palm.time = time;
 	t->thumb.state = THUMB_STATE_MAYBE;
@@ -320,9 +322,6 @@ tp_process_absolute(struct tp_dispatch *tp,
 	case ABS_MT_SLOT:
 		tp->slot = e->value;
 		break;
-	case ABS_MT_DISTANCE:
-		t->distance = e->value;
-		break;
 	case ABS_MT_TRACKING_ID:
 		if (e->value != -1)
 			tp_new_touch(tp, t, time);
@@ -363,6 +362,11 @@ tp_process_absolute_st(struct tp_dispatch *tp,
 		t->dirty = true;
 		tp->queued |= TOUCHPAD_EVENT_MOTION;
 		break;
+	case ABS_PRESSURE:
+		t->pressure = e->value;
+		t->dirty = true;
+		tp->queued |= TOUCHPAD_EVENT_OTHERAXIS;
+		break;
 	}
 }
 
@@ -543,8 +547,8 @@ tp_palm_tap_is_palm(const struct tp_dispatch *tp, const struct tp_touch *t)
 	/* We're inside the left/right palm edge and not in one of the
 	 * software button areas */
 	if (t->point.y < tp->buttons.bottom_area.top_edge) {
-		log_debug(tp_libinput_context(tp),
-			  "palm: palm-tap detected\n");
+		evdev_log_debug(tp->device,
+				"palm: palm-tap detected\n");
 		return true;
 	}
 
@@ -574,8 +578,8 @@ tp_palm_detect_dwt_triggered(struct tp_dispatch *tp,
 		if (t->palm.time == 0 ||
 		    t->palm.time > tp->dwt.keyboard_last_press_time) {
 			t->palm.state = PALM_NONE;
-			log_debug(tp_libinput_context(tp),
-				  "palm: touch released, timeout after typing\n");
+			evdev_log_debug(tp->device,
+					"palm: touch released, timeout after typing\n");
 		}
 	}
 
@@ -602,8 +606,8 @@ tp_palm_detect_trackpoint_triggered(struct tp_dispatch *tp,
 		if (t->palm.time == 0 ||
 		    t->palm.time > tp->palm.trackpoint_last_event_time) {
 			t->palm.state = PALM_NONE;
-			log_debug(tp_libinput_context(tp),
-				  "palm: touch released, timeout after trackpoint\n");
+			evdev_log_debug(tp->device,
+				       "palm: touch released, timeout after trackpoint\n");
 		}
 	}
 
@@ -623,7 +627,7 @@ tp_palm_detect_move_out_of_edge(struct tp_dispatch *tp,
 	if (time < t->palm.time + PALM_TIMEOUT &&
 	    (t->point.x > tp->palm.left_edge && t->point.x < tp->palm.right_edge)) {
 		delta = device_delta(t->point, t->palm.first);
-		dirs = normalized_get_direction(tp_normalize_delta(tp, delta));
+		dirs = phys_get_direction(tp_phys_delta(tp, delta));
 		if ((dirs & DIRECTIONS) && !(dirs & ~DIRECTIONS))
 			return true;
 	}
@@ -673,7 +677,7 @@ tp_palm_detect(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 	if (t->palm.state == PALM_EDGE) {
 		if (tp_palm_detect_multifinger(tp, t, time)) {
 			t->palm.state = PALM_NONE;
-			log_debug(tp_libinput_context(tp),
+			evdev_log_debug(tp->device,
 				  "palm: touch released, multiple fingers\n");
 
 		/* If labelled a touch as palm, we unlabel as palm when
@@ -682,7 +686,7 @@ tp_palm_detect(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 		 */
 		} else if (tp_palm_detect_move_out_of_edge(tp, t, time)) {
 			t->palm.state = PALM_NONE;
-			log_debug(tp_libinput_context(tp),
+			evdev_log_debug(tp->device,
 				  "palm: touch released, out of edge zone\n");
 		}
 		return;
@@ -711,7 +715,7 @@ tp_palm_detect(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 	t->palm.first = t->point;
 
 out:
-	log_debug(tp_libinput_context(tp),
+	evdev_log_debug(tp->device,
 		  "palm: palm detected (%s)\n",
 		  t->palm.state == PALM_EDGE ? "edge" :
 		  t->palm.state == PALM_TYPING ? "typing" : "trackpoint");
@@ -751,12 +755,11 @@ tp_thumb_detect(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 		t->thumb.initial = t->point;
 	else if (t->state == TOUCH_UPDATE) {
 		struct device_float_coords delta;
-		struct normalized_coords normalized;
+		struct phys_coords mm;
 
 		delta = device_delta(t->point, t->thumb.initial);
-		normalized = tp_normalize_delta(tp, delta);
-		if (normalized_length(normalized) >
-			TP_MM_TO_DPI_NORMALIZED(7)) {
+		mm = tp_phys_delta(tp, delta);
+		if (length_in_mm(mm) > 7) {
 			t->thumb.state = THUMB_STATE_NO;
 			goto out;
 		}
@@ -787,33 +790,79 @@ tp_thumb_detect(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 	 */
 out:
 	if (t->thumb.state != state)
-		log_debug(tp_libinput_context(tp),
+		evdev_log_debug(tp->device,
 			  "thumb state: %s → %s\n",
 			  thumb_state_to_str(state),
 			  thumb_state_to_str(t->thumb.state));
 }
 
 static void
-tp_unhover_abs_distance(struct tp_dispatch *tp, uint64_t time)
+tp_unhover_pressure(struct tp_dispatch *tp, uint64_t time)
 {
 	struct tp_touch *t;
-	unsigned int i;
+	int i;
+	unsigned int nfake_touches;
+	unsigned int real_fingers_down = 0;
 
-	for (i = 0; i < tp->ntouches; i++) {
+	nfake_touches = tp_fake_finger_count(tp);
+	if (nfake_touches == FAKE_FINGER_OVERFLOW)
+		nfake_touches = 0;
+
+	for (i = 0; i < (int)tp->num_slots; i++) {
 		t = tp_get_touch(tp, i);
 
-		if (!t->dirty)
-			continue;
-
-		if (t->state == TOUCH_HOVERING) {
-			if (t->distance == 0) {
-				/* avoid jumps when landing a finger */
-				tp_motion_history_reset(t);
-				tp_begin_touch(tp, t, time);
+		if (t->dirty) {
+			if (t->state == TOUCH_HOVERING) {
+				if (t->pressure >= tp->pressure.high) {
+					/* avoid jumps when landing a finger */
+					tp_motion_history_reset(t);
+					tp_begin_touch(tp, t, time);
+				}
+			} else {
+				if (t->pressure < tp->pressure.low)
+					tp_end_touch(tp, t, time);
 			}
-		} else {
-			if (t->distance > 0)
-				tp_end_touch(tp, t, time);
+		}
+
+		if (t->state == TOUCH_BEGIN ||
+		    t->state == TOUCH_UPDATE)
+			real_fingers_down++;
+	}
+
+	if (nfake_touches <= tp->num_slots ||
+	    tp->nfingers_down == 0)
+		return;
+
+	/* if we have more fake fingers down than slots, we assume
+	 * _all_ fingers have enough pressure, even if some of the slotted
+	 * ones don't. Anything else gets insane quickly.
+	 */
+	for (i = 0; i < (int)tp->ntouches; i++) {
+		t = tp_get_touch(tp, i);
+		if (t->state == TOUCH_HOVERING) {
+			/* avoid jumps when landing a finger */
+			tp_motion_history_reset(t);
+			tp_begin_touch(tp, t, time);
+
+			if (tp->nfingers_down >= nfake_touches)
+				break;
+		}
+	}
+
+	if (tp->nfingers_down > nfake_touches ||
+	    real_fingers_down == 0) {
+		for (i = tp->ntouches - 1; i >= 0; i--) {
+			t = tp_get_touch(tp, i);
+
+			if (t->state == TOUCH_HOVERING ||
+			    t->state == TOUCH_NONE)
+				continue;
+
+			tp_end_touch(tp, t, time);
+
+			if (real_fingers_down > 0  &&
+			    tp->nfingers_down == nfake_touches)
+				break;
 		}
 	}
 }
@@ -880,8 +929,8 @@ tp_unhover_fake_touches(struct tp_dispatch *tp, uint64_t time)
 static void
 tp_unhover_touches(struct tp_dispatch *tp, uint64_t time)
 {
-	if (tp->reports_distance)
-		tp_unhover_abs_distance(tp, time);
+	if (tp->pressure.use_pressure)
+		tp_unhover_pressure(tp, time);
 	else
 		tp_unhover_fake_touches(tp, time);
 
@@ -913,8 +962,8 @@ tp_position_fake_touches(struct tp_dispatch *tp)
 	}
 
 	if (!topmost) {
-		log_bug_libinput(tp_libinput_context(tp),
-				 "Unable to find topmost touch\n");
+		evdev_log_bug_libinput(tp->device,
+				       "Unable to find topmost touch\n");
 		return;
 	}
 
@@ -925,6 +974,7 @@ tp_position_fake_touches(struct tp_dispatch *tp)
 			continue;
 
 		t->point = topmost->point;
+		t->pressure = topmost->pressure;
 		if (!t->dirty)
 			t->dirty = topmost->dirty;
 	}
@@ -1028,7 +1078,7 @@ tp_process_state(struct tp_dispatch *tp, uint64_t time)
 
 		if (tp_detect_jumps(tp, t)) {
 			if (!tp->semi_mt)
-				log_bug_kernel(tp_libinput_context(tp),
+				evdev_log_bug_kernel(tp->device,
 					       "Touch jump detected and discarded.\n"
 					       "See %stouchpad_jumping_cursor.html for details\n",
 					       HTTP_DOC_LINK);
@@ -1141,8 +1191,7 @@ tp_interface_process(struct evdev_dispatch *dispatch,
 		     struct input_event *e,
 		     uint64_t time)
 {
-	struct tp_dispatch *tp =
-		(struct tp_dispatch *)dispatch;
+	struct tp_dispatch *tp = tp_dispatch(dispatch);
 
 	if (tp->ignore_events)
 		return;
@@ -1177,13 +1226,16 @@ tp_remove_sendevents(struct tp_dispatch *tp)
 	if (tp->dwt.keyboard)
 		libinput_device_remove_event_listener(
 					&tp->dwt.keyboard_listener);
+
+	if (tp->lid_switch.lid_switch)
+		libinput_device_remove_event_listener(
+					&tp->lid_switch.lid_switch_listener);
 }
 
 static void
 tp_interface_remove(struct evdev_dispatch *dispatch)
 {
-	struct tp_dispatch *tp =
-		(struct tp_dispatch*)dispatch;
+	struct tp_dispatch *tp = tp_dispatch(dispatch);
 
 	tp_remove_tap(tp);
 	tp_remove_buttons(tp);
@@ -1195,8 +1247,7 @@ tp_interface_remove(struct evdev_dispatch *dispatch)
 static void
 tp_interface_destroy(struct evdev_dispatch *dispatch)
 {
-	struct tp_dispatch *tp =
-		(struct tp_dispatch*)dispatch;
+	struct tp_dispatch *tp = tp_dispatch(dispatch);
 
 	free(tp->touches);
 	free(tp);
@@ -1258,7 +1309,7 @@ static void
 tp_interface_suspend(struct evdev_dispatch *dispatch,
 		     struct evdev_device *device)
 {
-	struct tp_dispatch *tp = (struct tp_dispatch *)dispatch;
+	struct tp_dispatch *tp = tp_dispatch(dispatch);
 
 	tp_clear_state(tp);
 }
@@ -1326,7 +1377,7 @@ tp_keyboard_timeout(uint64_t now, void *data)
 		libinput_timer_set(&tp->dwt.keyboard_timer,
 				   now + DEFAULT_KEYBOARD_ACTIVITY_TIMEOUT_2);
 		tp->dwt.keyboard_last_press_time = now;
-		log_debug(tp_libinput_context(tp), "palm: keyboard timeout refresh\n");
+		evdev_log_debug(tp->device, "palm: keyboard timeout refresh\n");
 		return;
 	}
 
@@ -1334,7 +1385,7 @@ tp_keyboard_timeout(uint64_t now, void *data)
 
 	tp->dwt.keyboard_active = false;
 
-	log_debug(tp_libinput_context(tp), "palm: keyboard timeout\n");
+	evdev_log_debug(tp->device, "palm: keyboard timeout\n");
 }
 
 static inline bool
@@ -1435,23 +1486,6 @@ tp_keyboard_event(uint64_t time, struct libinput_event *event, void *data)
 			   time + timeout);
 }
 
-static bool
-tp_dwt_device_is_blacklisted(struct evdev_device *device)
-{
-	unsigned int bus = libevdev_get_id_bustype(device->evdev);
-	unsigned int vendor_id = libevdev_get_id_vendor(device->evdev);
-
-	/* evemu will set the right bus type */
-	if (bus == BUS_VIRTUAL || bus == BUS_BLUETOOTH)
-		return true;
-
-	/* Wacom doesn't have devices that need dwt */
-	if (vendor_id == VENDOR_ID_WACOM)
-		return true;
-
-	return false;
-}
-
 static bool
 tp_want_dwt(struct evdev_device *touchpad,
 	    struct evdev_device *keyboard)
@@ -1463,10 +1497,6 @@ tp_want_dwt(struct evdev_device *touchpad,
 	unsigned int product_tp = evdev_device_get_id_product(touchpad);
 	unsigned int product_kbd = evdev_device_get_id_product(keyboard);
 
-	if (tp_dwt_device_is_blacklisted(touchpad) ||
-	    tp_dwt_device_is_blacklisted(keyboard))
-		return false;
-
 	/* External touchpads with the same vid/pid as the keyboard are
 	   considered a happy couple */
 	if (touchpad->tags & EVDEV_TAG_EXTERNAL_TOUCHPAD)
@@ -1520,10 +1550,10 @@ tp_dwt_pair_keyboard(struct evdev_device *touchpad,
 	tp->dwt.keyboard = keyboard;
 	tp->dwt.keyboard_active = false;
 
-	log_debug(touchpad->base.seat->libinput,
-		  "palm: dwt activated with %s<->%s\n",
-		  touchpad->devname,
-		  keyboard->devname);
+	evdev_log_debug(touchpad,
+			"palm: dwt activated with %s<->%s\n",
+			touchpad->devname,
+			keyboard->devname);
 }
 
 static void
@@ -1553,6 +1583,50 @@ tp_pair_trackpoint(struct evdev_device *touchpad,
 	}
 }
 
+static void
+tp_lid_switch_event(uint64_t time, struct libinput_event *event, void *data)
+{
+	struct tp_dispatch *tp = data;
+	struct libinput_event_switch *swev;
+
+	if (libinput_event_get_type(event) != LIBINPUT_EVENT_SWITCH_TOGGLE)
+		return;
+
+	swev = libinput_event_get_switch_event(event);
+	switch (libinput_event_switch_get_switch_state(swev)) {
+	case LIBINPUT_SWITCH_STATE_OFF:
+		tp_resume(tp, tp->device);
+		evdev_log_debug(tp->device, "lid: resume touchpad\n");
+		break;
+	case LIBINPUT_SWITCH_STATE_ON:
+		tp_suspend(tp, tp->device);
+		evdev_log_debug(tp->device, "lid: suspend touchpad\n");
+		break;
+	}
+}
+
+static void
+tp_pair_lid_switch(struct evdev_device *touchpad,
+		   struct evdev_device *lid_switch)
+{
+	struct tp_dispatch *tp = (struct tp_dispatch*)touchpad->dispatch;
+
+	if ((lid_switch->tags & EVDEV_TAG_LID_SWITCH) == 0)
+		return;
+
+	if (tp->lid_switch.lid_switch == NULL) {
+		evdev_log_debug(touchpad,
+				"lid_switch: activated for %s<->%s\n",
+				touchpad->devname,
+				lid_switch->devname);
+
+		libinput_device_add_event_listener(&lid_switch->base,
+					&tp->lid_switch.lid_switch_listener,
+					tp_lid_switch_event, tp);
+		tp->lid_switch.lid_switch = lid_switch;
+	}
+}
+
 static void
 tp_interface_device_added(struct evdev_device *device,
 			  struct evdev_device *added_device)
@@ -1561,6 +1635,7 @@ tp_interface_device_added(struct evdev_device *device,
 
 	tp_pair_trackpoint(device, added_device);
 	tp_dwt_pair_keyboard(device, added_device);
+	tp_pair_lid_switch(device, added_device);
 
 	if (tp->sendevents.current_mode !=
 	    LIBINPUT_CONFIG_SEND_EVENTS_DISABLED_ON_EXTERNAL_MOUSE)
@@ -1600,7 +1675,7 @@ tp_interface_device_removed(struct evdev_device *device,
 		return;
 
 	list_for_each(dev, &device->base.seat->devices_list, link) {
-		struct evdev_device *d = (struct evdev_device*)dev;
+		struct evdev_device *d = evdev_device(dev);
 		if (d != removed_device &&
 		    (d->tags & EVDEV_TAG_EXTERNAL_MOUSE)) {
 			return;
@@ -1641,10 +1716,9 @@ evdev_tag_touchpad(struct evdev_device *device,
 			evdev_tag_touchpad_external(device);
 			return;
 		} else {
-			log_info(evdev_libinput_context(device),
-				 "%s: tagged as unknown value %s\n",
-				 device->devname,
-				 prop);
+			evdev_log_info(device,
+				       "tagged with unknown value %s\n",
+				       prop);
 		}
 	}
 
@@ -1682,9 +1756,8 @@ evdev_tag_touchpad(struct evdev_device *device,
 
 	if ((device->tags &
 	    (EVDEV_TAG_EXTERNAL_TOUCHPAD|EVDEV_TAG_INTERNAL_TOUCHPAD)) == 0) {
-		log_bug_libinput(evdev_libinput_context(device),
-				 "%s: Internal or external? Please file a bug.\n",
-				 device->devname);
+		evdev_log_bug_libinput(device,
+				       "Internal or external? Please file a bug.\n");
 		evdev_tag_touchpad_external(device);
 	}
 }
@@ -1694,7 +1767,7 @@ tp_interface_toggle_touch(struct evdev_dispatch *dispatch,
 			  struct evdev_device *device,
 			  bool enable)
 {
-	struct tp_dispatch *tp = (struct tp_dispatch*)dispatch;
+	struct tp_dispatch *tp = tp_dispatch(dispatch);
 	bool ignore_events = !enable;
 
 	if (ignore_events == tp->ignore_events)
@@ -1746,7 +1819,23 @@ tp_sync_touch(struct tp_dispatch *tp,
 				       &t->point.y))
 		t->point.y = libevdev_get_event_value(evdev, EV_ABS, ABS_Y);
 
-	libevdev_fetch_slot_value(evdev, slot, ABS_MT_DISTANCE, &t->distance);
+	if (!libevdev_fetch_slot_value(evdev,
+				       slot,
+				       ABS_MT_PRESSURE,
+				       &t->pressure))
+		t->pressure = libevdev_get_event_value(evdev,
+						       EV_ABS,
+						       ABS_PRESSURE);
+}
+
+static inline void
+tp_disable_abs_mt(struct evdev_device *device)
+{
+	struct libevdev *evdev = device->evdev;
+	unsigned int code;
+
+	for (code = ABS_MT_SLOT; code <= ABS_MAX; code++)
+		libevdev_disable_event_code(evdev, EV_ABS, code);
 }
 
 static bool
@@ -1804,6 +1893,9 @@ tp_init_slots(struct tp_dispatch *tp,
 		tp->has_mt = false;
 	}
 
+	if (!tp->has_mt)
+		tp_disable_abs_mt(device);
+
 	ARRAY_FOR_EACH(max_touches, m) {
 		if (libevdev_has_event_code(device->evdev,
 					    EV_KEY,
@@ -1924,7 +2016,7 @@ tp_scroll_get_methods(struct tp_dispatch *tp)
 static uint32_t
 tp_scroll_config_scroll_method_get_methods(struct libinput_device *device)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct tp_dispatch *tp = (struct tp_dispatch*)evdev->dispatch;
 
 	return tp_scroll_get_methods(tp);
@@ -1934,7 +2026,7 @@ static enum libinput_config_status
 tp_scroll_config_scroll_method_set_method(struct libinput_device *device,
 		        enum libinput_config_scroll_method method)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct tp_dispatch *tp = (struct tp_dispatch*)evdev->dispatch;
 	uint64_t time = libinput_now(tp_libinput_context(tp));
 
@@ -1952,7 +2044,7 @@ tp_scroll_config_scroll_method_set_method(struct libinput_device *device,
 static enum libinput_config_scroll_method
 tp_scroll_config_scroll_method_get_method(struct libinput_device *device)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct tp_dispatch *tp = (struct tp_dispatch*)evdev->dispatch;
 
 	return tp->scroll.method;
@@ -1972,16 +2064,16 @@ tp_scroll_get_default_method(struct tp_dispatch *tp)
 		method = LIBINPUT_CONFIG_SCROLL_EDGE;
 
 	if ((methods & method) == 0)
-		log_bug_libinput(tp_libinput_context(tp),
-				 "Invalid default scroll method %d\n",
-				 method);
+		evdev_log_bug_libinput(tp->device,
+				       "invalid default scroll method %d\n",
+				       method);
 	return method;
 }
 
 static enum libinput_config_scroll_method
 tp_scroll_config_scroll_method_get_default_method(struct libinput_device *device)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct tp_dispatch *tp = (struct tp_dispatch*)evdev->dispatch;
 
 	return tp_scroll_get_default_method(tp);
@@ -2016,7 +2108,7 @@ static enum libinput_config_status
 tp_dwt_config_set(struct libinput_device *device,
 	   enum libinput_config_dwt_state enable)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct tp_dispatch *tp = (struct tp_dispatch*)evdev->dispatch;
 
 	switch(enable) {
@@ -2035,7 +2127,7 @@ tp_dwt_config_set(struct libinput_device *device,
 static enum libinput_config_dwt_state
 tp_dwt_config_get(struct libinput_device *device)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct tp_dispatch *tp = (struct tp_dispatch*)evdev->dispatch;
 
 	return tp->dwt.dwt_enabled ?
@@ -2052,7 +2144,7 @@ tp_dwt_default_enabled(struct tp_dispatch *tp)
 static enum libinput_config_dwt_state
 tp_dwt_config_get_default(struct libinput_device *device)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct tp_dispatch *tp = (struct tp_dispatch*)evdev->dispatch;
 
 	return tp_dwt_default_enabled(tp) ?
@@ -2060,11 +2152,27 @@ tp_dwt_config_get_default(struct libinput_device *device)
 		LIBINPUT_CONFIG_DWT_DISABLED;
 }
 
+static inline bool
+tp_is_tpkb_combo_below(struct evdev_device *device)
+{
+	const char *prop;
+	enum tpkbcombo_layout layout = TPKBCOMBO_LAYOUT_UNKNOWN;
+
+	prop = udev_device_get_property_value(device->udev_device,
+					      "LIBINPUT_ATTR_TPKBCOMBO_LAYOUT");
+	if (!prop)
+		return false;
+
+	return parse_tpkbcombo_layout_poperty(prop, &layout) &&
+		layout == TPKBCOMBO_LAYOUT_BELOW;
+}
+
 static void
 tp_init_dwt(struct tp_dispatch *tp,
 	    struct evdev_device *device)
 {
-	if (tp_dwt_device_is_blacklisted(device))
+	if (device->tags & EVDEV_TAG_EXTERNAL_TOUCHPAD &&
+	    !tp_is_tpkb_combo_below(device))
 		return;
 
 	tp->dwt.config.is_available = tp_dwt_config_is_available;
@@ -2088,8 +2196,8 @@ tp_init_palmdetect(struct tp_dispatch *tp,
 	tp->palm.right_edge = INT_MAX;
 	tp->palm.left_edge = INT_MIN;
 
-	/* Wacom doesn't have internal touchpads */
-	if (device->model_flags & EVDEV_MODEL_WACOM_TOUCHPAD)
+	if (device->tags & EVDEV_TAG_EXTERNAL_TOUCHPAD &&
+	    !tp_is_tpkb_combo_below(device))
 		return;
 
 	evdev_device_get_size(device, &width, &height);
@@ -2178,10 +2286,9 @@ tp_init_thumb(struct tp_dispatch *tp)
 	tp->thumb.threshold = max(100, threshold);
 
 out:
-	log_debug(tp_libinput_context(tp),
-		  "thumb: enabled thumb detection%s on '%s'\n",
-		  tp->thumb.threshold != INT_MAX ? " (+pressure)" : "",
-		  device->devname);
+	evdev_log_debug(device,
+			"thumb: enabled thumb detection%s\n",
+			tp->thumb.threshold != INT_MAX ? " (+pressure)" : "");
 }
 
 static bool
@@ -2189,7 +2296,6 @@ tp_pass_sanity_check(struct tp_dispatch *tp,
 		     struct evdev_device *device)
 {
 	struct libevdev *evdev = device->evdev;
-	struct libinput *libinput = tp_libinput_context(tp);
 
 	if (!libevdev_has_event_code(evdev, EV_ABS, ABS_X))
 		goto error;
@@ -2203,9 +2309,8 @@ tp_pass_sanity_check(struct tp_dispatch *tp,
 	return true;
 
 error:
-	log_bug_kernel(libinput,
-		       "device %s failed touchpad sanity checks\n",
-		       device->devname);
+	evdev_log_bug_kernel(device,
+			     "device failed touchpad sanity checks\n");
 	return false;
 }
 
@@ -2229,11 +2334,10 @@ tp_init_default_resolution(struct tp_dispatch *tp,
 	 * are old ones, so let's assume a small touchpad size and assume
 	 * that.
 	 */
-	log_info(tp_libinput_context(tp),
-		 "%s: no resolution or size hints, assuming a size of %dx%dmm\n",
-		 device->devname,
-		 touchpad_width_mm,
-		 touchpad_height_mm);
+	evdev_log_info(device,
+		       "no resolution or size hints, assuming a size of %dx%dmm\n",
+		       touchpad_width_mm,
+		       touchpad_height_mm);
 
 	xres = device->abs.dimensions.x/touchpad_width_mm;
 	yres = device->abs.dimensions.y/touchpad_height_mm;
@@ -2253,14 +2357,52 @@ tp_init_hysteresis(struct tp_dispatch *tp)
 	res_y = tp->device->abs.absinfo_y->resolution;
 	tp->hysteresis_margin.x = res_x/2;
 	tp->hysteresis_margin.y = res_y/2;
+}
 
-	return;
+static void
+tp_init_pressure(struct tp_dispatch *tp,
+		 struct evdev_device *device)
+{
+	const struct input_absinfo *abs;
+	unsigned int range;
+	unsigned int code = ABS_PRESSURE;
+
+	if (tp->has_mt)
+		code = ABS_MT_PRESSURE;
+
+	if (!libevdev_has_event_code(device->evdev, EV_ABS, code)) {
+		tp->pressure.use_pressure = false;
+		return;
+	}
+
+	tp->pressure.use_pressure = true;
+
+	abs = libevdev_get_abs_info(device->evdev, code);
+	assert(abs);
+
+	range = abs->maximum - abs->minimum;
+
+	if (device->model_flags & EVDEV_MODEL_ELANTECH_TOUCHPAD) {
+		tp->pressure.high = 24;
+		tp->pressure.low = 10;
+	} else if (device->model_flags & EVDEV_MODEL_CYAPA) {
+		tp->pressure.high = 10;
+		tp->pressure.low = 8;
+	} else {
+		/* Approximately the synaptics defaults */
+		tp->pressure.high = abs->minimum + 0.12 * range;
+		tp->pressure.low = abs->minimum + 0.10 * range;
+	}
+
+	evdev_log_debug(device,
+			"using pressure-based touch detection\n");
 }
 
 static int
 tp_init(struct tp_dispatch *tp,
 	struct evdev_device *device)
 {
+	tp->base.dispatch_type = DISPATCH_TOUCHPAD;
 	tp->base.interface = &tp_interface;
 	tp->device = device;
 
@@ -2274,9 +2416,7 @@ tp_init(struct tp_dispatch *tp,
 
 	evdev_device_init_abs_range_warnings(device);
 
-	tp->reports_distance = libevdev_has_event_code(device->evdev,
-						       EV_ABS,
-						       ABS_MT_DISTANCE);
+	tp_init_pressure(tp, device);
 
 	/* Set the dpi to that of the x axis, because that's what we normalize
 	   to when needed*/
@@ -2306,7 +2446,7 @@ tp_init(struct tp_dispatch *tp,
 static uint32_t
 tp_sendevents_get_modes(struct libinput_device *device)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	uint32_t modes = LIBINPUT_CONFIG_SEND_EVENTS_DISABLED;
 
 	if (evdev->tags & EVDEV_TAG_INTERNAL_TOUCHPAD)
@@ -2322,7 +2462,7 @@ tp_suspend_conditional(struct tp_dispatch *tp,
 	struct libinput_device *dev;
 
 	list_for_each(dev, &device->base.seat->devices_list, link) {
-		struct evdev_device *d = (struct evdev_device*)dev;
+		struct evdev_device *d = evdev_device(dev);
 		if (d->tags & EVDEV_TAG_EXTERNAL_MOUSE) {
 			tp_suspend(tp, device);
 			return;
@@ -2334,7 +2474,7 @@ static enum libinput_config_status
 tp_sendevents_set_mode(struct libinput_device *device,
 		       enum libinput_config_send_events_mode mode)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct tp_dispatch *tp = (struct tp_dispatch*)evdev->dispatch;
 
 	/* DISABLED overrides any DISABLED_ON_ */
@@ -2367,7 +2507,7 @@ tp_sendevents_set_mode(struct libinput_device *device,
 static enum libinput_config_send_events_mode
 tp_sendevents_get_mode(struct libinput_device *device)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct tp_dispatch *dispatch = (struct tp_dispatch*)evdev->dispatch;
 
 	return dispatch->sendevents.current_mode;
