@@ -94,6 +94,11 @@ struct phys_coords {
 	double y;
 };
 
+/* A pair of tilt flags */
+struct wheel_tilt_flags {
+	bool vertical, horizontal;
+};
+
 struct tablet_axes {
 	struct device_coords point;
 	struct normalized_coords delta;
@@ -461,6 +466,9 @@ void
 libinput_device_set_device_group(struct libinput_device *device,
 				 struct libinput_device_group *group);
 
+void
+libinput_device_init_event_listener(struct libinput_event_listener *listener);
+
 void
 libinput_device_add_event_listener(struct libinput_device *device,
 				   struct libinput_event_listener *listener,
@@ -618,6 +626,11 @@ tablet_pad_notify_strip(struct libinput_device *device,
 			double value,
 			enum libinput_tablet_pad_strip_axis_source source,
 			struct libinput_tablet_pad_mode_group *group);
+void
+switch_notify_toggle(struct libinput_device *device,
+		     uint64_t time,
+		     enum libinput_switch sw,
+		     enum libinput_switch_state state);
 
 static inline uint64_t
 libinput_now(struct libinput *libinput)
@@ -694,6 +707,12 @@ normalized_is_zero(struct normalized_coords norm)
 	return norm.x == 0.0 && norm.y == 0.0;
 }
 
+static inline double
+length_in_mm(struct phys_coords mm)
+{
+	return hypot(mm.x, mm.y);
+}
+
 enum directions {
 	N  = 1 << 0,
 	NE = 1 << 1,
@@ -752,9 +771,9 @@ xy_get_direction(double x, double y)
 }
 
 static inline uint32_t
-normalized_get_direction(struct normalized_coords norm)
+phys_get_direction(struct phys_coords mm)
 {
-	return xy_get_direction(norm.x, norm.y);
+	return xy_get_direction(mm.x, mm.y);
 }
 
 /**
