@@ -763,6 +763,20 @@ START_TEST(device_context)
 }
 END_TEST
 
+START_TEST(device_user_data)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+	void *userdata = &dev; /* not referenced */
+
+	ck_assert(libinput_device_get_user_data(device) == NULL);
+	libinput_device_set_user_data(device, userdata);
+	ck_assert_ptr_eq(libinput_device_get_user_data(device), userdata);
+	libinput_device_set_user_data(device, NULL);
+	ck_assert(libinput_device_get_user_data(device) == NULL);
+}
+END_TEST
+
 static int open_restricted(const char *path, int flags, void *data)
 {
 	int fd;
@@ -1468,6 +1482,119 @@ START_TEST(device_quirks_apple_magicmouse)
 }
 END_TEST
 
+START_TEST(device_quirks_logitech_marble_mouse)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+
+	litest_drain_events(li);
+
+	ck_assert(!libinput_device_pointer_has_button(dev->libinput_device,
+						      BTN_MIDDLE));
+}
+END_TEST
+
+START_TEST(device_capability_at_least_one)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+	enum libinput_device_capability caps[] = {
+		LIBINPUT_DEVICE_CAP_KEYBOARD,
+		LIBINPUT_DEVICE_CAP_POINTER,
+		LIBINPUT_DEVICE_CAP_TOUCH,
+		LIBINPUT_DEVICE_CAP_TABLET_TOOL,
+		LIBINPUT_DEVICE_CAP_TABLET_PAD,
+		LIBINPUT_DEVICE_CAP_GESTURE,
+		LIBINPUT_DEVICE_CAP_SWITCH,
+	};
+	enum libinput_device_capability *cap;
+	int ncaps = 0;
+
+	ARRAY_FOR_EACH(caps, cap) {
+		if (libinput_device_has_capability(device, *cap))
+			ncaps++;
+	}
+	ck_assert_int_gt(ncaps, 0);
+
+}
+END_TEST
+
+START_TEST(device_capability_check_invalid)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+
+	ck_assert(!libinput_device_has_capability(device, -1));
+	ck_assert(!libinput_device_has_capability(device, 7));
+	ck_assert(!libinput_device_has_capability(device, 0xffff));
+
+}
+END_TEST
+
+START_TEST(device_has_size)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+	double w, h;
+	int rc;
+
+	rc = libinput_device_get_size(device, &w, &h);
+	ck_assert_int_eq(rc, 0);
+	/* This matches the current set of test devices but may fail if
+	 * newer ones are added */
+	ck_assert_double_gt(w, 40);
+	ck_assert_double_gt(h, 20);
+}
+END_TEST
+
+START_TEST(device_has_no_size)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+	double w = 45, h = 67;
+	int rc;
+
+	rc = libinput_device_get_size(device, &w, &h);
+	ck_assert_int_eq(rc, -1);
+	ck_assert_double_eq(w, 45);
+	ck_assert_double_eq(h, 67);
+}
+END_TEST
+
+START_TEST(device_get_output)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+	const char *output_name;
+
+	output_name = libinput_device_get_output_name(device);
+	ck_assert_str_eq(output_name, "myOutput");
+}
+END_TEST
+
+START_TEST(device_no_output)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+	const char *output_name;
+
+	output_name = libinput_device_get_output_name(device);
+	ck_assert(output_name == NULL);
+}
+END_TEST
+
+START_TEST(device_seat_phys_name)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+	struct libinput_seat *seat = libinput_device_get_seat(device);
+	const char *seat_name;
+
+	seat_name = libinput_seat_get_physical_name(seat);
+	ck_assert(streq(seat_name, "seat0"));
+}
+END_TEST
+
 void
 litest_setup_tests_device(void)
 {
@@ -1498,6 +1625,7 @@ litest_setup_tests_device(void)
 	litest_add("device:sendevents", device_disable_topsoftbutton, LITEST_TOPBUTTONPAD, LITEST_ANY);
 	litest_add("device:id", device_ids, LITEST_ANY, LITEST_ANY);
 	litest_add_for_device("device:context", device_context, LITEST_SYNAPTICS_CLICKPAD_X220);
+	litest_add_for_device("device:context", device_user_data, LITEST_SYNAPTICS_CLICKPAD_X220);
 
 	litest_add("device:udev", device_get_udev_handle, LITEST_ANY, LITEST_ANY);
 
@@ -1532,4 +1660,19 @@ litest_setup_tests_device(void)
 	litest_add_for_device("device:quirks", device_quirks_no_abs_mt_y, LITEST_ANKER_MOUSE_KBD);
 	litest_add_for_device("device:quirks", device_quirks_cyborg_rat_mode_button, LITEST_CYBORG_RAT);
 	litest_add_for_device("device:quirks", device_quirks_apple_magicmouse, LITEST_MAGICMOUSE);
+	litest_add_for_device("device:quirks", device_quirks_logitech_marble_mouse, LITEST_LOGITECH_TRACKBALL);
+
+	litest_add("device:capability", device_capability_at_least_one, LITEST_ANY, LITEST_ANY);
+	litest_add("device:capability", device_capability_check_invalid, LITEST_ANY, LITEST_ANY);
+
+	litest_add("device:size", device_has_size, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add("device:size", device_has_size, LITEST_TABLET, LITEST_ANY);
+	litest_add("device:size", device_has_no_size, LITEST_ANY,
+		   LITEST_TOUCHPAD|LITEST_TABLET|LITEST_TOUCH|LITEST_ABSOLUTE|LITEST_SINGLE_TOUCH);
+
+	litest_add_for_device("device:output", device_get_output, LITEST_CALIBRATED_TOUCHSCREEN);
+	litest_add("device:output", device_no_output, LITEST_RELATIVE, LITEST_ANY);
+	litest_add("device:output", device_no_output, LITEST_KEYS, LITEST_ANY);
+
+	litest_add("device:seat", device_seat_phys_name, LITEST_ANY, LITEST_ANY);
 }
