@@ -45,6 +45,9 @@ static const uint32_t screen_width = 100;
 static const uint32_t screen_height = 100;
 struct tools_context context;
 static unsigned int stop = 0;
+static bool be_quiet = false;
+
+#define printq(...) ({ if (!be_quiet)  printf(__VA_ARGS__); })
 
 static void
 print_event_header(struct libinput_event *ev)
@@ -133,11 +136,14 @@ print_event_header(struct libinput_event *ev)
 	case LIBINPUT_EVENT_TABLET_PAD_STRIP:
 		type = "TABLET_PAD_STRIP";
 		break;
+	case LIBINPUT_EVENT_SWITCH_TOGGLE:
+		type = "SWITCH_TOGGLE";
+		break;
 	}
 
 	prefix = (last_device != dev) ? '-' : ' ';
 
-	printf("%c%-7s  %-16s ",
+	printq("%c%-7s  %-16s ",
 	       prefix,
 	       libinput_device_get_sysname(dev),
 	       type);
@@ -148,96 +154,54 @@ print_event_header(struct libinput_event *ev)
 static void
 print_event_time(uint32_t time)
 {
-	printf("%+6.2fs	", (time - start_time) / 1000.0);
+	printq("%+6.2fs	", (time - start_time) / 1000.0);
 }
 
-static void
-print_device_notify(struct libinput_event *ev)
+static inline void
+print_device_options(struct libinput_device *dev)
 {
-	struct libinput_device *dev = libinput_event_get_device(ev);
-	struct libinput_seat *seat = libinput_device_get_seat(dev);
-	struct libinput_device_group *group;
-	double w, h;
 	uint32_t scroll_methods, click_methods;
-	static int next_group_id = 0;
-	intptr_t group_id;
-
-	group = libinput_device_get_device_group(dev);
-	group_id = (intptr_t)libinput_device_group_get_user_data(group);
-	if (!group_id) {
-		group_id = ++next_group_id;
-		libinput_device_group_set_user_data(group, (void*)group_id);
-	}
-
-	printf("%-33s %5s %7s group%-2d",
-	       libinput_device_get_name(dev),
-	       libinput_seat_get_physical_name(seat),
-	       libinput_seat_get_logical_name(seat),
-	       (int)group_id);
-
-	printf(" cap:");
-	if (libinput_device_has_capability(dev,
-					   LIBINPUT_DEVICE_CAP_KEYBOARD))
-		printf("k");
-	if (libinput_device_has_capability(dev,
-					   LIBINPUT_DEVICE_CAP_POINTER))
-		printf("p");
-	if (libinput_device_has_capability(dev,
-					   LIBINPUT_DEVICE_CAP_TOUCH))
-		printf("t");
-	if (libinput_device_has_capability(dev,
-					   LIBINPUT_DEVICE_CAP_GESTURE))
-		printf("g");
-	if (libinput_device_has_capability(dev,
-					   LIBINPUT_DEVICE_CAP_TABLET_TOOL))
-		printf("T");
-	if (libinput_device_has_capability(dev,
-					   LIBINPUT_DEVICE_CAP_TABLET_PAD))
-		printf("P");
-
-	if (libinput_device_get_size(dev, &w, &h) == 0)
-		printf("  size %.0fx%.0fmm", w, h);
 
 	if (libinput_device_config_tap_get_finger_count(dev)) {
-	    printf(" tap");
+	    printq(" tap");
 	    if (libinput_device_config_tap_get_drag_lock_enabled(dev))
-		    printf("(dl on)");
+		    printq("(dl on)");
 	    else
-		    printf("(dl off)");
+		    printq("(dl off)");
 	}
 	if (libinput_device_config_left_handed_is_available(dev))
-	    printf(" left");
+	    printq(" left");
 	if (libinput_device_config_scroll_has_natural_scroll(dev))
-	    printf(" scroll-nat");
+	    printq(" scroll-nat");
 	if (libinput_device_config_calibration_has_matrix(dev))
-	    printf(" calib");
+	    printq(" calib");
 
 	scroll_methods = libinput_device_config_scroll_get_methods(dev);
 	if (scroll_methods != LIBINPUT_CONFIG_SCROLL_NO_SCROLL) {
-		printf(" scroll");
+		printq(" scroll");
 		if (scroll_methods & LIBINPUT_CONFIG_SCROLL_2FG)
-			printf("-2fg");
+			printq("-2fg");
 		if (scroll_methods & LIBINPUT_CONFIG_SCROLL_EDGE)
-			printf("-edge");
+			printq("-edge");
 		if (scroll_methods & LIBINPUT_CONFIG_SCROLL_ON_BUTTON_DOWN)
-			printf("-button");
+			printq("-button");
 	}
 
 	click_methods = libinput_device_config_click_get_methods(dev);
 	if (click_methods != LIBINPUT_CONFIG_CLICK_METHOD_NONE) {
-		printf(" click");
+		printq(" click");
 		if (click_methods & LIBINPUT_CONFIG_CLICK_METHOD_BUTTON_AREAS)
-			printf("-buttonareas");
+			printq("-buttonareas");
 		if (click_methods & LIBINPUT_CONFIG_CLICK_METHOD_CLICKFINGER)
-			printf("-clickfinger");
+			printq("-clickfinger");
 	}
 
 	if (libinput_device_config_dwt_is_available(dev)) {
 		if (libinput_device_config_dwt_get_enabled(dev) ==
 		    LIBINPUT_CONFIG_DWT_ENABLED)
-			printf(" dwt-on");
+			printq(" dwt-on");
 		else
-			printf(" dwt-off)");
+			printq(" dwt-off)");
 	}
 
 	if (libinput_device_has_capability(dev,
@@ -249,14 +213,67 @@ print_device_notify(struct libinput_event *ev)
 		nrings = libinput_device_tablet_pad_get_num_rings(dev);
 		ngroups = libinput_device_tablet_pad_get_num_mode_groups(dev);
 
-		printf(" buttons:%d strips:%d rings:%d mode groups:%d",
+		printq(" buttons:%d strips:%d rings:%d mode groups:%d",
 		       nbuttons,
 		       nstrips,
 		       nrings,
 		       ngroups);
 	}
+}
 
-	printf("\n");
+static void
+print_device_notify(struct libinput_event *ev)
+{
+	struct libinput_device *dev = libinput_event_get_device(ev);
+	struct libinput_seat *seat = libinput_device_get_seat(dev);
+	struct libinput_device_group *group;
+	double w, h;
+	static int next_group_id = 0;
+	intptr_t group_id;
+
+	group = libinput_device_get_device_group(dev);
+	group_id = (intptr_t)libinput_device_group_get_user_data(group);
+	if (!group_id) {
+		group_id = ++next_group_id;
+		libinput_device_group_set_user_data(group, (void*)group_id);
+	}
+
+	printq("%-33s %5s %7s group%-2d",
+	       libinput_device_get_name(dev),
+	       libinput_seat_get_physical_name(seat),
+	       libinput_seat_get_logical_name(seat),
+	       (int)group_id);
+
+	printq(" cap:");
+	if (libinput_device_has_capability(dev,
+					   LIBINPUT_DEVICE_CAP_KEYBOARD))
+		printq("k");
+	if (libinput_device_has_capability(dev,
+					   LIBINPUT_DEVICE_CAP_POINTER))
+		printq("p");
+	if (libinput_device_has_capability(dev,
+					   LIBINPUT_DEVICE_CAP_TOUCH))
+		printq("t");
+	if (libinput_device_has_capability(dev,
+					   LIBINPUT_DEVICE_CAP_GESTURE))
+		printq("g");
+	if (libinput_device_has_capability(dev,
+					   LIBINPUT_DEVICE_CAP_TABLET_TOOL))
+		printq("T");
+	if (libinput_device_has_capability(dev,
+					   LIBINPUT_DEVICE_CAP_TABLET_PAD))
+		printq("P");
+	if (libinput_device_has_capability(dev,
+					   LIBINPUT_DEVICE_CAP_SWITCH))
+		printq("S");
+
+	if (libinput_device_get_size(dev, &w, &h) == 0)
+		printq("  size %.0fx%.0fmm", w, h);
+
+	if (libinput_event_get_type(ev) == LIBINPUT_EVENT_DEVICE_ADDED)
+		print_device_options(dev);
+
+	printq("\n");
 
 }
 
@@ -285,7 +302,7 @@ print_key_event(struct libinput *li, struct libinput_event *ev)
 		keyname = libevdev_event_code_get_name(EV_KEY, key);
 		keyname = keyname ? keyname : "???";
 	}
-	printf("%s (%d) %s\n",
+	printq("%s (%d) %s\n",
 	       keyname,
 	       key,
 	       state == LIBINPUT_KEY_STATE_PRESSED ? "pressed" : "released");
@@ -300,7 +317,7 @@ print_motion_event(struct libinput_event *ev)
 
 	print_event_time(libinput_event_pointer_get_time(p));
 
-	printf("%6.2f/%6.2f\n", x, y);
+	printq("%6.2f/%6.2f\n", x, y);
 }
 
 static void
@@ -313,7 +330,7 @@ print_absmotion_event(struct libinput_event *ev)
 		p, screen_height);
 
 	print_event_time(libinput_event_pointer_get_time(p));
-	printf("%6.2f/%6.2f\n", x, y);
+	printq("%6.2f/%6.2f\n", x, y);
 }
 
 static void
@@ -330,7 +347,7 @@ print_pointer_button_event(struct libinput_event *ev)
 	buttonname = libevdev_event_code_get_name(EV_KEY, button);
 
 	state = libinput_event_pointer_get_button_state(p);
-	printf("%s (%d) %s, seat count: %u\n",
+	printq("%s (%d) %s, seat count: %u\n",
 	       buttonname ? buttonname : "???",
 	       button,
 	       state == LIBINPUT_BUTTON_STATE_PRESSED ? "pressed" : "released",
@@ -346,7 +363,7 @@ print_tablet_tip_event(struct libinput_event *ev)
 	print_event_time(libinput_event_tablet_tool_get_time(p));
 
 	state = libinput_event_tablet_tool_get_tip_state(p);
-	printf("%s\n", state == LIBINPUT_TABLET_TOOL_TIP_DOWN ? "down" : "up");
+	printq("%s\n", state == LIBINPUT_TABLET_TOOL_TIP_DOWN ? "down" : "up");
 }
 
 static void
@@ -363,7 +380,7 @@ print_tablet_button_event(struct libinput_event *ev)
 	buttonname = libevdev_event_code_get_name(EV_KEY, button);
 
 	state = libinput_event_tablet_tool_get_button_state(p);
-	printf("%3d (%s) %s, seat count: %u\n",
+	printq("%3d (%s) %s, seat count: %u\n",
 	       button,
 	       buttonname ? buttonname : "???",
 	       state == LIBINPUT_BUTTON_STATE_PRESSED ? "pressed" : "released",
@@ -389,6 +406,9 @@ print_pointer_axis_event(struct libinput_event *ev)
 	case LIBINPUT_POINTER_AXIS_SOURCE_CONTINUOUS:
 		source = "continuous";
 		break;
+	case LIBINPUT_POINTER_AXIS_SOURCE_WHEEL_TILT:
+		source = "tilt";
+		break;
 	}
 
 	if (libinput_event_pointer_has_axis(p,
@@ -404,7 +424,7 @@ print_pointer_axis_event(struct libinput_event *ev)
 		have_horiz = "*";
 	}
 	print_event_time(libinput_event_pointer_get_time(p));
-	printf("vert %.2f%s horiz %.2f%s (%s)\n",
+	printq("vert %.2f%s horiz %.2f%s (%s)\n",
 	       v, have_vert, h, have_horiz, source);
 }
 
@@ -422,14 +442,14 @@ print_tablet_axes(struct libinput_event_tablet_tool *t)
 
 	x = libinput_event_tablet_tool_get_x(t);
 	y = libinput_event_tablet_tool_get_y(t);
-	printf("\t%.2f%s/%.2f%s",
+	printq("\t%.2f%s/%.2f%s",
 	       x, changed_sym(t, x),
 	       y, changed_sym(t, y));
 
 	if (libinput_tablet_tool_has_tilt(tool)) {
 		x = libinput_event_tablet_tool_get_tilt_x(t);
 		y = libinput_event_tablet_tool_get_tilt_y(t);
-		printf("\ttilt: %.2f%s/%.2f%s",
+		printq("\ttilt: %.2f%s/%.2f%s",
 		       x, changed_sym(t, tilt_x),
 		       y, changed_sym(t, tilt_y));
 	}
@@ -439,29 +459,29 @@ print_tablet_axes(struct libinput_event_tablet_tool *t)
 		dist = libinput_event_tablet_tool_get_distance(t);
 		pressure = libinput_event_tablet_tool_get_pressure(t);
 		if (dist)
-			printf("\tdistance: %.2f%s",
+			printq("\tdistance: %.2f%s",
 			       dist, changed_sym(t, distance));
 		else
-			printf("\tpressure: %.2f%s",
+			printq("\tpressure: %.2f%s",
 			       pressure, changed_sym(t, pressure));
 	}
 
 	if (libinput_tablet_tool_has_rotation(tool)) {
 		rotation = libinput_event_tablet_tool_get_rotation(t);
-		printf("\trotation: %.2f%s",
+		printq("\trotation: %.2f%s",
 		       rotation, changed_sym(t, rotation));
 	}
 
 	if (libinput_tablet_tool_has_slider(tool)) {
 		slider = libinput_event_tablet_tool_get_slider_position(t);
-		printf("\tslider: %.2f%s",
+		printq("\tslider: %.2f%s",
 		       slider, changed_sym(t, slider));
 	}
 
 	if (libinput_tablet_tool_has_wheel(tool)) {
 		wheel = libinput_event_tablet_tool_get_wheel_delta(t);
 		delta = libinput_event_tablet_tool_get_wheel_delta_discrete(t);
-		printf("\twheel: %.2f%s (%d)",
+		printq("\twheel: %.2f%s (%d)",
 		       wheel, changed_sym(t, wheel),
 		       (int)delta);
 	}
@@ -474,7 +494,7 @@ print_tablet_axis_event(struct libinput_event *ev)
 
 	print_event_time(libinput_event_tablet_tool_get_time(t));
 	print_tablet_axes(t);
-	printf("\n");
+	printq("\n");
 }
 
 static void
@@ -483,7 +503,7 @@ print_touch_event_without_coords(struct libinput_event *ev)
 	struct libinput_event_touch *t = libinput_event_get_touch_event(ev);
 
 	print_event_time(libinput_event_touch_get_time(t));
-	printf("\n");
+	printq("\n");
 }
 
 static void
@@ -529,51 +549,53 @@ print_proximity_event(struct libinput_event *ev)
 		print_tablet_axes(t);
 		state_str = "proximity-in";
 	} else if (state == LIBINPUT_TABLET_TOOL_PROXIMITY_STATE_OUT) {
+		print_tablet_axes(t);
 		state_str = "proximity-out";
-		printf("\t");
 	} else {
 		abort();
 	}
 
-	printf("\t%s (%#" PRIx64 ", id %#" PRIx64 ") %s",
+	printq("\t%s (%#" PRIx64 ", id %#" PRIx64 ") %s",
 	       tool_str,
 	       libinput_tablet_tool_get_serial(tool),
 	       libinput_tablet_tool_get_tool_id(tool),
 	       state_str);
 
-	printf("\taxes:");
-	if (libinput_tablet_tool_has_distance(tool))
-		printf("d");
-	if (libinput_tablet_tool_has_pressure(tool))
-		printf("p");
-	if (libinput_tablet_tool_has_tilt(tool))
-		printf("t");
-	if (libinput_tablet_tool_has_rotation(tool))
-		printf("r");
-	if (libinput_tablet_tool_has_slider(tool))
-		printf("s");
-	if (libinput_tablet_tool_has_wheel(tool))
-		printf("w");
+	if (state == LIBINPUT_TABLET_TOOL_PROXIMITY_STATE_IN) {
+		printq("\taxes:");
+		if (libinput_tablet_tool_has_distance(tool))
+			printq("d");
+		if (libinput_tablet_tool_has_pressure(tool))
+			printq("p");
+		if (libinput_tablet_tool_has_tilt(tool))
+			printq("t");
+		if (libinput_tablet_tool_has_rotation(tool))
+			printq("r");
+		if (libinput_tablet_tool_has_slider(tool))
+			printq("s");
+		if (libinput_tablet_tool_has_wheel(tool))
+			printq("w");
 
-	printf("\tbtn:");
-	if (libinput_tablet_tool_has_button(tool, BTN_TOUCH))
-		printf("T");
-	if (libinput_tablet_tool_has_button(tool, BTN_STYLUS))
-		printf("S");
-	if (libinput_tablet_tool_has_button(tool, BTN_STYLUS2))
-		printf("S2");
-	if (libinput_tablet_tool_has_button(tool, BTN_LEFT))
-		printf("L");
-	if (libinput_tablet_tool_has_button(tool, BTN_MIDDLE))
-		printf("M");
-	if (libinput_tablet_tool_has_button(tool, BTN_RIGHT))
-		printf("R");
-	if (libinput_tablet_tool_has_button(tool, BTN_SIDE))
-		printf("Sd");
-	if (libinput_tablet_tool_has_button(tool, BTN_EXTRA))
-		printf("Ex");
+		printq("\tbtn:");
+		if (libinput_tablet_tool_has_button(tool, BTN_TOUCH))
+			printq("T");
+		if (libinput_tablet_tool_has_button(tool, BTN_STYLUS))
+			printq("S");
+		if (libinput_tablet_tool_has_button(tool, BTN_STYLUS2))
+			printq("S2");
+		if (libinput_tablet_tool_has_button(tool, BTN_LEFT))
+			printq("L");
+		if (libinput_tablet_tool_has_button(tool, BTN_MIDDLE))
+			printq("M");
+		if (libinput_tablet_tool_has_button(tool, BTN_RIGHT))
+			printq("R");
+		if (libinput_tablet_tool_has_button(tool, BTN_SIDE))
+			printq("Sd");
+		if (libinput_tablet_tool_has_button(tool, BTN_EXTRA))
+			printq("Ex");
+	}
 
-	printf("\n");
+	printq("\n");
 }
 
 static void
@@ -587,7 +609,7 @@ print_touch_event_with_coords(struct libinput_event *ev)
 
 	print_event_time(libinput_event_touch_get_time(t));
 
-	printf("%d (%d) %5.2f/%5.2f (%5.2f/%5.2fmm)\n",
+	printq("%d (%d) %5.2f/%5.2f (%5.2f/%5.2fmm)\n",
 	       libinput_event_touch_get_slot(t),
 	       libinput_event_touch_get_seat_slot(t),
 	       x, y,
@@ -609,7 +631,7 @@ print_gesture_event_without_coords(struct libinput_event *ev)
 	    cancelled = libinput_event_gesture_get_cancelled(t);
 
 	print_event_time(libinput_event_gesture_get_time(t));
-	printf("%d%s\n", finger_count, cancelled ? " cancelled" : "");
+	printq("%d%s\n", finger_count, cancelled ? " cancelled" : "");
 }
 
 static void
@@ -623,7 +645,7 @@ print_gesture_event_with_coords(struct libinput_event *ev)
 
 	print_event_time(libinput_event_gesture_get_time(t));
 
-	printf("%d %5.2f/%5.2f (%5.2f/%5.2f unaccelerated)",
+	printq("%d %5.2f/%5.2f (%5.2f/%5.2f unaccelerated)",
 	       libinput_event_gesture_get_finger_count(t),
 	       dx, dy, dx_unaccel, dy_unaccel);
 
@@ -632,9 +654,9 @@ print_gesture_event_with_coords(struct libinput_event *ev)
 		double scale = libinput_event_gesture_get_scale(t);
 		double angle = libinput_event_gesture_get_angle_delta(t);
 
-		printf(" %5.2f @ %5.2f\n", scale, angle);
+		printq(" %5.2f @ %5.2f\n", scale, angle);
 	} else {
-		printf("\n");
+		printq("\n");
 	}
 }
 
@@ -651,16 +673,16 @@ print_tablet_pad_button_event(struct libinput_event *ev)
 	button = libinput_event_tablet_pad_get_button_number(p),
 	state = libinput_event_tablet_pad_get_button_state(p);
 	mode = libinput_event_tablet_pad_get_mode(p);
-	printf("%3d %s (mode %d)",
+	printq("%3d %s (mode %d)",
 	       button,
 	       state == LIBINPUT_BUTTON_STATE_PRESSED ? "pressed" : "released",
 	       mode);
 
 	group = libinput_event_tablet_pad_get_mode_group(p);
 	if (libinput_tablet_pad_mode_group_button_is_toggle(group, button))
-		printf(" <mode toggle>");
+		printq(" <mode toggle>");
 
-	printf("\n");
+	printq("\n");
 }
 
 static void
@@ -682,7 +704,7 @@ print_tablet_pad_ring_event(struct libinput_event *ev)
 	}
 
 	mode = libinput_event_tablet_pad_get_mode(p);
-	printf("ring %d position %.2f (source %s) (mode %d)\n",
+	printq("ring %d position %.2f (source %s) (mode %d)\n",
 	       libinput_event_tablet_pad_get_ring_number(p),
 	       libinput_event_tablet_pad_get_ring_position(p),
 	       source,
@@ -708,13 +730,35 @@ print_tablet_pad_strip_event(struct libinput_event *ev)
 	}
 
 	mode = libinput_event_tablet_pad_get_mode(p);
-	printf("strip %d position %.2f (source %s) (mode %d)\n",
+	printq("strip %d position %.2f (source %s) (mode %d)\n",
 	       libinput_event_tablet_pad_get_strip_number(p),
 	       libinput_event_tablet_pad_get_strip_position(p),
 	       source,
 	       mode);
 }
 
+static void
+print_switch_event(struct libinput_event *ev)
+{
+	struct libinput_event_switch *sw = libinput_event_get_switch_event(ev);
+	enum libinput_switch_state state;
+	const char *which;
+
+	print_event_time(libinput_event_switch_get_time(sw));
+
+	switch (libinput_event_switch_get_switch(sw)) {
+	case LIBINPUT_SWITCH_LID:
+		which = "lid";
+		break;
+	default:
+		abort();
+	}
+
+	state = libinput_event_switch_get_switch_state(sw);
+
+	printq("switch %s state %d\n", which, state);
+}
+
 static int
 handle_and_print_events(struct libinput *li)
 {
@@ -803,6 +847,9 @@ handle_and_print_events(struct libinput *li)
 		case LIBINPUT_EVENT_TABLET_PAD_STRIP:
 			print_tablet_pad_strip_event(ev);
 			break;
+		case LIBINPUT_EVENT_SWITCH_TOGGLE:
+			print_switch_event(ev);
+			break;
 		}
 
 		libinput_event_destroy(ev);
@@ -861,6 +908,8 @@ main(int argc, char **argv)
 	if (tools_parse_args(argc, argv, &context))
 		return 1;
 
+	be_quiet = context.options.quiet;
+
 	li = tools_open_backend(&context);
 	if (!li)
 		return 1;
